<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\LeachWaterColorReport;
use Modules\Qc\Entities\LeachWaterColorReportAttachment;

class LeachWaterColorReportController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('leach_water_color'), $filename);

            $leach_water_color_attachment = new LeachWaterColorReportAttachment();
            $leach_water_color_attachment->leach_water_color_id = 0;
            $leach_water_color_attachment->path = $filename;
            $leach_water_color_attachment->token = $token;
            $leach_water_color_attachment->save();

            $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_leach_water_color_reports.*,
            qc_leach_water_color_report_attachments.path,
            hr_employee_sites.site
        FROM
            qc_leach_water_color_report_attachments
            INNER JOIN
            qc_leach_water_color_reports
            ON
                qc_leach_water_color_report_attachments.leach_water_color_id = qc_leach_water_color_reports.leach_water_color_id      
            INNER JOIN
            hr_employee_sites
            ON
                qc_leach_water_color_reports.factory = hr_employee_sites.site_id
                GROUP BY
            qc_leach_water_color_reports.leach_water_color_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "leach_water_color_id" => $record->leach_water_color_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $leach = new LeachWaterColorReport();
            $leach->user_id = Auth::user()->id;
            $leach->reference_no = $request->get('reference_no');
            $leach->date = $request->get('date');
            $leach->factory = $request->get('factory');
            $leach->description = $request->get('description');
            if ($leach->save()) {
                $token = $request->get('attachment_token');
                $attachment = LeachWaterColorReportAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->leach_water_color_id = $leach->leach_water_color_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $leach = LeachWaterColorReport::find($id);
            $leach->user_id = Auth::user()->id;
            $leach->reference_no = $request->get('reference_no');
            $leach->date = $request->get('date');
            $leach->factory = $request->get('factory');
            $leach->description = $request->get('description');
            if ($leach->update()) {
                $token = $request->get('attachment_token');
                $attachments = LeachWaterColorReportAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = LeachWaterColorReportAttachment::where('token', '=', $token)->first();
                        $attachment->leach_water_color_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = LeachWaterColorReportAttachment::where('leach_water_color_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = LeachWaterColorReport::where('leach_water_color_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = LeachWaterColorReportAttachment::where('leach_water_color_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_leach_water_color_reports.*,
            qc_leach_water_color_report_attachments.token
        FROM
            qc_leach_water_color_report_attachments
            INNER JOIN
            qc_leach_water_color_reports
            ON
                qc_leach_water_color_report_attachments.leach_water_color_id = qc_leach_water_color_reports.leach_water_color_id
        WHERE
            qc_leach_water_color_reports.leach_water_color_id = {$id}";

            $leach =  DB::select($query);

            $responseBody = $this->responseBody(true, "LeachWaterColorReportController", "getData", $leach[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "LeachWaterColorReportController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
