<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\PcmSupplierVisitFormat;
use Modules\Qc\Entities\PcmSupplierVisitFormatAttachment;

class PcmSupplierVisitFormatController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('pcm_supplier_visit'), $filename);

            $pcm_supplier_visit_attachment = new PcmSupplierVisitFormatAttachment();
            $pcm_supplier_visit_attachment->pcm_supplier_visit_id = 0;
            $pcm_supplier_visit_attachment->path = $filename;
            $pcm_supplier_visit_attachment->token = $token;
            $pcm_supplier_visit_attachment->save();

            $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allRecords()
    {
        try {
            $query = "SELECT
            qc_pcm_supplier_visit_formats.*,
            qc_pcm_supplier_visit_format_attachments.path,
            hr_employee_sites.site
        FROM
            qc_pcm_supplier_visit_format_attachments
            INNER JOIN
            qc_pcm_supplier_visit_formats
            ON
                qc_pcm_supplier_visit_format_attachments.pcm_supplier_visit_id = qc_pcm_supplier_visit_formats.pcm_supplier_visit_id           
            INNER JOIN
            hr_employee_sites
            ON
                qc_pcm_supplier_visit_formats.factory = hr_employee_sites.site_id
                GROUP BY
            qc_pcm_supplier_visit_formats.pcm_supplier_visit_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "pcm_supplier_visit_id" => $record->pcm_supplier_visit_id,
                    "reference_no" => $record->reference_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveRecord(Request $request)
    {
        try {
            $pcm = new PcmSupplierVisitFormat();
            $pcm->user_id = Auth::user()->id;
            $pcm->reference_no = $request->get('reference_no');
            $pcm->date = $request->get('date');
            $pcm->factory = $request->get('factory');
            $pcm->description = $request->get('description');
            if ($pcm->save()) {
                $token = $request->get('attachment_token');
                $attachment = PcmSupplierVisitFormatAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->pcm_supplier_visit_id = $pcm->pcm_supplier_visit_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "saveRecord", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $pcm = PcmSupplierVisitFormat::find($id);
            $pcm->user_id = Auth::user()->id;
            $pcm->reference_no = $request->get('reference_no');
            $pcm->date = $request->get('date');
            $pcm->factory = $request->get('factory');
            $pcm->description = $request->get('description');
            if ($pcm->update()) {
                $token = $request->get('attachment_token');
                $attachments = PcmSupplierVisitFormatAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = PcmSupplierVisitFormatAttachment::where('token', '=', $token)->first();
                        $attachment->pcm_supplier_visit_id = (int) $id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = PcmSupplierVisitFormatAttachment::where('pcm_supplier_visit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = PcmSupplierVisitFormat::where('pcm_supplier_visit_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = PcmSupplierVisitFormatAttachment::where('pcm_supplier_visit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getData($id)
    {
        try {
            $query = "SELECT
            qc_pcm_supplier_visit_formats.*,
            qc_pcm_supplier_visit_format_attachments.token
        FROM
            qc_pcm_supplier_visit_format_attachments
            INNER JOIN
            qc_pcm_supplier_visit_formats
            ON
                qc_pcm_supplier_visit_format_attachments.pcm_supplier_visit_id = qc_pcm_supplier_visit_formats.pcm_supplier_visit_id
        WHERE
            qc_pcm_supplier_visit_formats.pcm_supplier_visit_id = {$id}";

            $pcm = DB::select($query);

            $responseBody = $this->responseBody(true, "PcmSupplierVisitFormatController", "getData", $pcm[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PcmSupplierVisitFormatController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}